.form-control-dark {
  border-color: var(--bs-gray);
}
.form-control-dark:focus {
  border-color: #fff;
  box-shadow: 0 0 0 .25rem rgba(255, 255, 255, .25);
}

.text-small {
  font-size: 85%;
}

.dropdown-toggle:not(:focus) {
  outline: 0;
}

/* Ensure dropdown aligns with search bar */
.dropdown-menu {
  width: 100%;
  max-height: 200px;
  overflow-y: auto;
  display: none;
}

.dropdown-menu.show {
  display: block;
}

#graph-container svg {
  transition: transform 0.3s ease;
}